<?php
/*
 * Plugin Name: WP-Cat2Calendar
 * Author URI: http://codeispoetry.ru/
 * Plugin URI: http://codeispoetry.ru/?page_id=52
 * Description: Simple plugin which allows make a calendar from posts in selected category.
 * Author: Andrew Mihaylov
 * Version: 1.0
 * $Id: wp-cat2calendar.php 169183 2009-11-01 07:59:47Z andddd $
 * Tags: calendar, post, category, future post, organize, agenda
 */

/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

define('WP_CAT2CALENDAR_TEXTDOMAIN', 'wp-cat2calendar');

add_action('init', 'wp_cat2calendar_init');
add_action('admin_menu', 'wp_cat2calendar_menu');
add_action('permalink_structure_changed', 'wp_cat2calendar_permalinks_changed');
add_action('admin_post_wp-cat2calendar-settings', 'wp_cat2calendar_settings');

add_filter('the_posts', 'wp_cat2calendar_future_posts');
add_filter('query_vars', 'wp_cat2calendar_query_vars');
add_filter('generate_rewrite_rules', 'wp_cat2calendar_rewrite');

add_shortcode('WP-Cat2Calendar', 'wp_cat2calendar');

register_activation_hook( __FILE__, 'wp_cat2calendar_activate');

if(function_exists('register_uninstall_hook'))
    register_uninstall_hook(__FILE__, 'wp_cat2calendar_uninstall');

function wp_cat2calendar_activate()
{
    global $wp_rewrite;

    $wp_rewrite->flush_rules();

    add_option('wp_cat2cal_use_default_css', true);
    add_option('wp_cat2cal_use_permalinks', $wp_rewrite->using_permalinks());
}

function wp_cat2calendar_uninstall()
{
    delete_option('wp_cat2cal_use_default_css');
    delete_option('wp_cat2cal_use_permalinks');
}

function wp_cat2calendar_expsvnprops($var_base)
{
    $revId = '';

    if(preg_match('/\d+/', '$Rev: 169183 $', $m))
        $revId = array_pop($m);

    define($var_base . '_REV', $revId);
}

function wp_cat2calendar_init()
{
    global $wp_version;

    wp_cat2calendar_expsvnprops('WP_CAT2CALENDAR');

    load_plugin_textdomain(WP_CAT2CALENDAR_TEXTDOMAIN, PLUGINDIR . '/' . dirname(plugin_basename(__FILE__)) . '/langs/', //2.5 Compatibility
                                           dirname(plugin_basename(__FILE__)) . '/langs/'); //2.6+, Works with custom wp-content dirs.

    if(version_compare($wp_version, '2.7', '<'))
    {
        add_action('wp_head', 'wp_cat2calendar_wphead');
    }
    else
    {
        add_action('template_redirect', 'wp_cat2calendar_templredirect');
    }
}

function wp_cat2calendar_wphead()
{
    echo '<link rel="stylesheet" href="' . WP_PLUGIN_URL . '/' . dirname(plugin_basename(__FILE__)) . '/default.css?ver=' . WP_CAT2CALENDAR_REV . '" type="text/css" media="all" />';
}

function wp_cat2calendar_templredirect()
{
    if((bool)get_option('wp_cat2cal_use_default_css') != false)
    {
        wp_enqueue_style('wp-cat2calendar-compat', WP_PLUGIN_URL . '/' . dirname(plugin_basename(__FILE__)) . '/default.css', array(), WP_CAT2CALENDAR_REV, 'all');
    }
}

function wp_cat2calendar_menu()
{
    $page = add_submenu_page( 'options-general.php', __('WP-Cat2Calendar', WP_CAT2CALENDAR_TEXTDOMAIN), __('WP-Cat2Calendar', WP_CAT2CALENDAR_TEXTDOMAIN), 7,  __FILE__, 'wp_cat2calendar_manage');
    add_action('load-' . $page, 'wp_cat2calendar_onload_manage');
    add_action('admin_print_styles-' . $page, 'wp_cat2calendar_admin_styles');
    if(function_exists('add_contextual_help'))
        add_contextual_help($page, __('<p><h5>Usage</h5></p>
<p>Use <tt>WP-Cat2Calendar</tt> shortcode in your post/page to add a calendar.</p>

<p>You can add a calendar using the php lines:</p>

<pre>$options = array(...);
echo wp_cat2calendar($options);</pre>


<p><strong>Options:</strong></p>


<ul>
    <li>cat_id – a comma separated list of category ID\'s (uncategorized by default)</li>
    <li>year – year you want to display in calendar (current year by default)</li>
    <li>month – month you want to display in calendar (current month by default)</li>
    <li>show_nav – show/hide month/year navigation, 0 or 1 (0 by default)</li>
    <li>show_date – show/hide selected month/year title, 0 or 1 (0 by default). <span style="text-decoration: underline;">Have no affect if navigation is shown</span>.</li>
    <li>allow_change_date – allow user to navigate through a calendar even if navigation is hidden and user has direct link. <span style="text-decoration: underline;">Has no affect if navigation is shown</span>.</li>
</ul>


<p><strong>Examples:</strong></p>

<p><tt>[WP-Cat2Calendar cat_id="3,4" show_nav="1" year="2009" month="10"]</tt></p>

<p>It will show a calendar of posts for WordPress categories with ID 3 and 4 with navigation and the start date for a calendar will be October, 2009.</p>

<p><tt>[WP-Cat2Calendar cat_id="1" show_nav="1"]</tt></p>

<p>It will show a calendar of posts for WordPress category ID 1 with navigation and the start date for a calendar will be current date.</p>', WP_CAT2CALENDAR_TEXTDOMAIN));
}

function wp_cat2calendar_settings()
{
    global $wp_rewrite;

    if ( !current_user_can('manage_options') )
        wp_die( __('Cheatin&#8217; uh?') );

    check_admin_referer('wp-cat2calendar-settings');

    $use_default_css = isset($_POST['use_default_css']);
    $use_permalinks = isset($_POST['use_permalinks']);

    update_option('wp_cat2cal_use_default_css', $use_default_css);
    update_option('wp_cat2cal_use_permalinks', $use_permalinks);

    $wp_rewrite->flush_rules();

    setcookie('_wp_cat2_calendar_saved', true);

    wp_redirect($_POST['_wp_http_referer']);
}

function wp_cat2calendar_onload_manage()
{
    if(isset($_COOKIE['_wp_cat2_calendar_saved']))
        setcookie('_wp_cat2_calendar_saved', 0, time()-2592000);
}

function wp_cat2calendar_admin_styles()
{
    global $wp_version;

    if(version_compare($wp_version, '2.8', '<'))
        wp_enqueue_style('wp-cat2calendar-compat', WP_PLUGIN_URL . '/' . dirname(plugin_basename(__FILE__)) . '/compat-wplt28.css', array(), WP_CAT2CALENDAR_REV, 'all');
}

function wp_cat2calendar_manage()
{
    global $wp_rewrite;
?>

<?php if(isset($_COOKIE['_wp_cat2_calendar_saved'])) : ?>
	<div class="updated fade"><p><strong><?php _e('Options updated.', WP_CAT2CALENDAR_TEXTDOMAIN); ?></strong></p></div>
<?php endif; ?>

    <div class="wrap wp_cat2calendar_wrap">
        <h2><? _e('WP-Cat2Calendar Settings', WP_CAT2CALENDAR_TEXTDOMAIN); ?></h2>

        <form method="post" action="admin-post.php">
            <?php wp_nonce_field('wp-cat2calendar-settings'); ?>
            <input type="hidden" name="action" value="wp-cat2calendar-settings" />

            <table class="form-table">
                <tr valign="top">
                    <th scope="row"><?php _e('Styles', WP_CAT2CALENDAR_TEXTDOMAIN); ?></th>
                    <td>
                        <fieldset>
                            <legend class="screen-reader-text"><span><?php _e('Styles', WP_CAT2CALENDAR_TEXTDOMAIN); ?></span></legend>
                            <label for="use_default_css">
                            <input name="use_default_css" type="checkbox" id="use_default_css" value="1"<?php if((bool)get_option('wp_cat2cal_use_default_css') != false) : ?> checked='checked'<?php endif; ?> />
                            <?php _e('Use default (bundled) CSS', WP_CAT2CALENDAR_TEXTDOMAIN); ?></label>
                        </fieldset>
                    </td>
                </tr>

                <tr valign="top">
                    <th scope="row"><?php _e('Use permalinks', WP_CAT2CALENDAR_TEXTDOMAIN); ?></th>
                    <td>
                        <fieldset>
                            <legend class="screen-reader-text"><span><?php _e('Use permalinks', WP_CAT2CALENDAR_TEXTDOMAIN); ?></span></legend>
                            <label for="use_permalinks">
                            <input name="use_permalinks" type="checkbox" id="use_permalinks" value="1"<?php if((bool)get_option('wp_cat2cal_use_permalinks') != false && $wp_rewrite->using_permalinks()) : ?> checked='checked'<?php endif; ?><?php if(!$wp_rewrite->using_permalinks()) : ?> disabled<?php endif; ?> />
                            <?php echo sprintf(__('URL will look like <tt>%1$s/my-post/%2$d-%3$d</tt> instead of <tt>%1$s/index.php?p=1&date=%2$d-%3$d</tt>', WP_CAT2CALENDAR_TEXTDOMAIN), get_bloginfo('wpurl'), date('Y'), date('m')); ?></label>
                        </fieldset>
                    </td>
                </tr>
            </table>


            <p class="submit">
                <input name="Submit" class="button-primary" value="<?php _e('Save Changes'); ?>" type="submit" />
            </p>
        </form>


    </div>
<?php
}

function wp_cat2calendar_permalinks_changed($value) {
    if(empty($value)) {
        update_option('wp_cat2cal_use_permalinks', false);
    }
}

function wp_cat2calendar_query_vars($vars) {
    array_push($vars, 'date');
    return $vars;
}

function wp_cat2calendar_rewrite($wp_rewrite) {
    $rules = array(
        '(.+?)/([0-9]{4}-[0-9]{2})$' => 'index.php?pagename=' . $wp_rewrite->preg_index(1) . '&date=' . $wp_rewrite->preg_index(2)
    );

    $wp_rewrite->rules = $rules + $wp_rewrite->rules;
}

/* Small hack which allows future posts to be shown but only if is_single */
function wp_cat2calendar_future_posts($posts)
{
   global $wp_query, $wpdb;

   if(is_single() && $wp_query->post_count == 0)
   {
      $posts = $wpdb->get_results($wp_query->request);
   }

   return $posts;
}

function wp_cat2calendar_permalink($date)
{
    global $wp_rewrite;

    $permalink = get_permalink();
    
    if($wp_rewrite->using_permalinks() &&
       (bool)get_option('wp_cat2cal_use_permalinks')) {
        return $permalink . '/' . $date;
    }

    if(preg_match('/[?](.*+)$/', $permalink))
        return $permalink . '&amp;date=' . $date;
    else
        return $permalink . '?date=' . $date;
}

function wp_cat2calendar_filter_date(&$month, &$year) {
    if((int)$month < 1)
        $month = 1;
    else if((int)$month > 12)
        $month = 12;
}

function wp_cat2calendar($atts=array())
{
    global $wp_locale;

    $query_date = get_query_var('date');
    $current_date = current_time('timestamp');
    $year = gmdate('Y', $current_date);
    $month = gmdate('m', $current_date);
    $time = gmmktime(0, 0, 0, $month, 1, $year);
    $wp_start_of_week = get_option('start_of_week');
    $calendar_days = array();
    
    $query = new WP_Query();
    $posts = array();

    // fill days order array depends on start of week wp option
    for($i = $wp_start_of_week; $i < 7; $i++)
        $calendar_days[] = $i;

    for($i = 0; $i < $wp_start_of_week; $i++)
        $calendar_days[] = $i;

    extract(shortcode_atts(
            array('cat_id' => 1, /* uncategorized by default */
                  'year' => $year,
                  'month' => $month,
                  'show_nav' => false,
                  'show_date' => false,
                  'allow_change_date' => false
            ),
            $atts));

    if(!empty($query_date) && ($show_nav || $allow_change_date))
    {
        $matches = array();
        if(preg_match('#(\d{4})-(\d{2})#', $query_date, $matches)) {
            $month = array_pop($matches);
            $year = array_pop($matches);
        }
    }

    /* filter cat_id */
    $cat_id = preg_replace(array('#[^0-9,]#', '#([,])+#', '#^[,]|[,]$#'), array('', '$1', ''), $cat_id);

    /* apply date modification if query_var or shortcode are set */
    wp_cat2calendar_filter_date($month, $year);
    $time = gmmktime(0, 0, 0, $month, 1, $year);
    $month = gmdate('m', $time);
    $year = gmdate('Y', $time);

    $num_days = date('t', $time);
    $first_day = date('w', $time);
    $day_n = 1;

    $prev_month = gmmktime(0, 0, 0, $month-1, 1, $year);
    $next_month = gmmktime(0, 0, 0, $month+1, 1, $year);

    $out = '<div class="wp_cat2calendar">';

    if($show_nav)
        $out .= '<div class="nav">
            <div class="left"><a href="' . wp_cat2calendar_permalink(gmdate('Y-m', $prev_month)) . '">&laquo; ' . gmdate('F, Y', $prev_month) . '</a></div>
            <div class="right"><a href="' . wp_cat2calendar_permalink(gmdate('Y-m', $next_month)) . '">' . gmdate('F, Y', $next_month) . ' &raquo;</a></div>
            <div class="center current_date">' . gmdate('F, Y', $time) . '</div>
        </div>
        ';
    else if($show_date)
        $out .= '<div class="nav">
            <div class="current_date">' . gmdate('F, Y', $time) . '</div>
        </div>';

    $out .= '<table>';
    $out .= '<tr>';

    for ($day_index = 0; $day_index <= 6; $day_index++) {
        $classes = array();
        $class = '';

        $classes[] = 'day-' . $day_index;
        
        if($day_index == 0) $classes[] = 'day-first';
        if($day_index == 6) $classes[] = 'day-last';

        if(!empty($classes))
            $class = ' class="' . implode(' ', $classes) . '"';
        
        $out .= '<th' . $class . '>' . $wp_locale->get_weekday($calendar_days[$day_index]) . '</th>';
    }

    $out .= '</tr>';

    for($i = 0; $i < sizeof($calendar_days); $i++)
        if($calendar_days[$i] == $first_day)
            $day_n -= $i;
  
    $table_rows = ceil(($num_days - $day_n + 1) / 7);
    $q_posts = $query->query('cat=' . $cat_id . '&post_status=future,publish&monthnum=' . $month . '&year=' . $year);
    

    for($i = 0; $i < sizeof($q_posts); $i++) {
        $day = mysql2date('j', $q_posts[$i]->post_date);

        if(!array_key_exists($day, $posts))
            $posts[$day] = array();

        $posts[$day][] =& $q_posts[$i];
    }

    for($i = 0; $i < $table_rows; $i++) { // walk by weeks
        $out .= '<tr>';

        for($j = 0; $j < 7; $j++) { // walk by days of week

            $css_classes = array();
            $css_classes[] = 'day-' . $j;

            if($j == 0) $css_classes[] = 'day-first';
            if($j == 6) $css_classes[] = 'day-last';

            if($day_n < 1 || $day_n > $num_days)
            {
                $css_classes[] = 'empty';
                $class = ' class="' . implode(' ', $css_classes) . '"';
                $out .= '<td' . $class . '><!-- --></td>';
            } 
            else
            {
                $class = '';
                $has_posts = array_key_exists($day_n, $posts);
                $is_more = $current_date > strtotime($day_n . '.' . $month . '.' . $year);
                $is_current = $day_n == date('j', $current_date) &&
                              $month == date('m', $current_date) &&
                              $year == date('Y', $current_date);

                if($has_posts)
                {
                    $css_classes[] = $is_current ? 'current' : ($is_more ? 'past' : 'future');
                } else {
                    $css_classes[] = 'no-posts';

                    if($is_current)
                        $css_classes[] = 'current';
                }

                if(!empty($css_classes))
                    $class = ' class="' . implode(' ', $css_classes) . '"';
                    
                $out .= '<td' . $class . '><span class="day">' . $day_n . '</span><br/>';
                
                if($has_posts)
                {
                    $out .= '<ul class="posts">';
                    foreach($posts[$day_n] as $post) {
                        $out .= '<li><a href="' . get_permalink($post->ID) . '">' . $post->post_title . '</a></li>';
                    }
                    $out .= '</ul>';
                }

                
                $out .= '</td>';
            }
            $day_n++;
        }
        $out .= '</tr>';
    }

    $out .= '</table>';
    $out .= '</div>';

    return $out;
}
?>